package nl.amis.jfall2009.ejb.session;

import java.util.Collections;
import java.util.List;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import nl.amis.jfall2009.ejb.entity.Constellation;
import nl.amis.jfall2009.ejb.entity.DeepSkyType;
import nl.amis.jfall2009.ejb.entity.Sac;

@Stateless
public class JFall2009SessionBean {

    @PersistenceContext
    private EntityManager em;

    private boolean whereAdded = false;

    /**
     * Returns a list of SAC objects
     *
     * @param name The catalog name of the SAC objects
     * @param ra1 A right ascension boundary of the SAC objects
     * @param ra2 A right ascension boundary of the SAC objects
     * @param dec1 A declination boundary of the SAC objects
     * @param dec2 A declination boundary of the SAC objects
     * @param mag A magnitude boundary of the SAC objects
     * @param constellation The constallation of the SAC objects
     * @param sacType The DeepSky Type of the SAC objects
     * @return a list of SAC objects that meet all criteria
     */
    @SuppressWarnings("all")
    public List<Sac> getSacObjects(String name, Integer ra1, Integer ra2, Integer dec1, Integer dec2, Double mag, String constellation, String sacType) {
        String sql = "select s from Sac s";
        sql = addNameSql(name, sql);
        sql = addRaSql(ra1, ra2, sql);
        sql = addDecSql(dec1, dec2, sql);
        sql = addMagSql(mag, sql);
        sql = addConstellationSql(constellation, sql);
        sql = setSacTypeSql(sacType, sql);
        sql += " order by s.raH, s.raM, s.deSign, s.deD, s.deM";
        Query query = em.createQuery(sql);
        setQueryParameters(query, name, ra1, ra2, dec1, dec2, mag, constellation, sacType);
        query.setMaxResults(200);
        List<Sac> sacList = query.getResultList();
        whereAdded = false;
        return sacList;
    }

    /**
     * Fetches all Catalogs from the database
     *
     * @return a list of Catalogs
     */
    public List<String> getCatalogs() {
        String sql1 = "select distinct s.sacObjectCatalog from Sac s where s.sacObjectCatalog is not null";
        String sql2 = "select distinct s.otherCatalog from Sac s where s.otherCatalog is not null";
        Query query = em.createQuery(sql1);
        List<String> catalogs = query.getResultList();
        query = em.createQuery(sql2);
        for (String catalog : (List<String>) query.getResultList()) {
            if (!catalogs.contains(catalog)) {
                catalogs.add(catalog);
            }
        }
        Collections.sort(catalogs);
        return catalogs;
    }

    private String addNameSql(String name, String sql) {
        if (name != null && !name.equals("")) {
            if (!whereAdded) { sql += " where "; whereAdded = true; } else { sql += " and "; }
            sql += "(s.sacObjectCatalog = :name or s.otherCatalog = :name)";
        }
        return sql;
    }

    private String addRaSql(Integer ra1, Integer ra2, String sql) {
        if (ra1 != null && ra2 != null) {
            if (!whereAdded) { sql += " where "; whereAdded = true; } else { sql += " and "; }
            if (ra1 <= ra2) {
                sql += "(s.raH >= :ra1 and s.raH <= :ra2)";
            } else {
                sql += "(s.raH >= :ra1 or s.raH <= :ra2)";
            }
        }
        return sql;
    }

    private String addDecSql(Integer dec1, Integer dec2, String sql) {
        if (dec1 != null && dec2 != null) {
            if (!whereAdded) { sql += " where "; whereAdded = true; } else { sql += " and "; }
            if (dec1 < dec2) {
                Integer temp = dec1;
                dec1 = dec2;
                dec2 = temp;
            }
            if (dec1 >=0 && dec2 >= 0) {
                sql += "(s.deSign = '+' and (s.deD <= :dec1 and s.deD >= :dec2))";
            } else if (dec1 <= 0 && dec2 <= 0) {
                sql += "(s.deSign = '-' and (s.deD >= :dec1 and s.deD <= :dec2))";
            } else {
                sql += "((s.deSign = '+' and s.deD <= :dec1) or (s.deSign = '-' and s.deD <= :dec2))";
            }
        }
        return sql;
    }

    private String addMagSql(Double mag, String sql) {
        if (mag != null) {
            if (!whereAdded) { sql += " where "; whereAdded = true; } else { sql += " and "; }
            sql += "s.mag <= :mag";
        }
        return sql;
    }

    private String addConstellationSql(String constellation, String sql) {
        if (constellation != null && !constellation.equals("")) {
            if (!whereAdded) { sql += " where "; whereAdded = true; } else { sql += " and "; }
            sql += "s.constellation.name = :constellation";
        }
        return sql;
    }

    private String setSacTypeSql(String sacType, String sql) {
        if (sacType != null && !sacType.equals("")) {
            if (!whereAdded) { sql += " where "; whereAdded = true; } else { sql += " and "; }
            sql += "s.sacType.name = :sacType";
        }
        return sql;
    }

    private void setQueryParameters(Query query, String name, Integer ra1, Integer ra2, Integer dec1, Integer dec2, Double mag, String constellation, String sacType) {
        if (name != null && !name.equals("")) {
            query.setParameter("name", name);
        }
        if (ra1 != null && ra2 != null) {
            query.setParameter("ra1", ra1);
            query.setParameter("ra2", ra2);
        }
        if (dec1 != null && dec2 != null) {
            query.setParameter("dec1", Math.abs(dec1));
            query.setParameter("dec2", Math.abs(dec2));
        }
        if (mag != null) {
            query.setParameter("mag", new Double(mag));
        }
        if (constellation != null && !constellation.equals("")) {
            query.setParameter("constellation", constellation);
        }
        if (sacType != null && !sacType.equals("")) {
            query.setParameter("sacType", sacType);
        }
    }

    /**
     * Saves a new Sac object to the database
     * @param sac The new Sac object
     */
    public void addSac(Sac sac) {
        em.persist(sac);
    }

    /**
     * Retrieves a Constellation from the database by its name.
     * @param name
     * @return
     */
    public Constellation getConstellationByName(String name) {
        Query query = em.createNamedQuery(Constellation.QUERY_FIND_CONSTELLATION_BY_NAME);
        query.setParameter("name", name);

        System.out.println("Looking for constellation with name " + name);

        try {
            return (Constellation) query.getSingleResult();
        } catch (NoResultException e) {
            return null;
        }
    }

    /**
     * Retrieves a DeepSkyType from the database by its name.
     * @param name
     * @return
     */
    public DeepSkyType getDeepSkyTypeByName(String name) {
        Query query = em.createNamedQuery(DeepSkyType.QUERY_FIND_DEEP_SKY_TYPE_BY_NAME);
        query.setParameter("name", name);
        try {
            return (DeepSkyType) query.getSingleResult();
        } catch (NoResultException e) {
            return null;
        }
    }

}
